#lang racket/base

; Utility to build a sitemap and provide links in headers
(require
  racket/string)

(provide
  sitemap
  build-headers)

(define *sitemap-list* '()) ; sitemap

; Site structure
(struct sitepage
  (name ; Name displayed in headers, or #f if the page should not be shown in the header (but still considered when displaying the headers, mostly relevant for second-level pages)
   path ; Absolute path to the page
   link? ; Indicate if the link is clickable when on the same page
  ))

; Function to set the sitemap
(define (sitemap lst)
  (set! *sitemap-list*
    (map (lambda (x) (apply sitepage x)) lst)))

; Navigation link
(define (navigation-link str-path page)
  (if (not (sitepage-name page))
      ""
      `(section ((class "nav-item"))
        ,(if (or (not (equal? str-path (sitepage-path page)))
                  (sitepage-link? page))
            `(a ((href ,(sitepage-path page))) ,(sitepage-name page))
            (sitepage-name page)))))

; Build the navigation header
(define (build-headers current-path)
  (let ((str-path (string-join current-path "/" #:before-first "/")))
    (map
      (lambda (x)
        (navigation-link str-path x))
      *sitemap-list*)))
