#lang racket/base

; Templates for the pages
(require
  "../webcontainer/weblets.rkt"
  "../webcontainer/weblet-parameter.rkt"
  "sitemap.rkt"
  racket/date
  )

(provide
  make-nav-header
  pages:template
  pages:adaptable-template)

; Make the header
(define (make-nav-header param)
  (define path (weblet-parameter-path param))
  `(header
    ; Current page
    (nav ,@(build-headers path))))

; Main template for pages
(define (pages:template
          #:title title ; Title
          #:content content ; Content, may be a function taking a weblet-parameter
          #:date [date #f] ; Date
          #:stylesheets [stylesheets (list)] ; Stylesheets to use on the page
          #:scripts [scripts (list)] ; Scripts to use of the page
          #:on-load [on-load #f] ; the onload attribute of the body of the page
          #:error-code [error-code 200] ; Error code returned
          )
  (html-page-weblet
    #:error-code error-code
    #:body
      (lambda (param)
        `(html
            (head
              (title ,title)
              (meta ((charset "UTF-8")))
              (link ((href "/css/beleth.css")(rel "stylesheet")(type "text/css")(media "all")))
              ,@(map
                (lambda (x)
                  `(link ((href ,x)(rel "stylesheet")(type "text/css")(media "all"))))
                stylesheets)
              ,@(map
                (lambda (x)
                  `(script ((src ,x)) " "))
                scripts)
              )
            (body (,@(if on-load `((onload ,on-load)) '()))
              ,(make-nav-header param)
              (main
                (h1 ,title)
                (hr)
                ,(apply-or-return content param))
              (footer ,(string-append "©2015-" (number->string (date-year (current-date))) " Feufochmar"))
              )))))

; Adaptable template
; Similar to pages:template, but the argument is a function taking the weblet parameters and returning a hash containing the arguments for pages:template
(define (pages:adaptable-template content)
  (lambda (param)
    (define result (content param))
    ((pages:template
       #:title (hash-ref result 'title "Untitled page")
       #:content (hash-ref result 'content)
       #:date (hash-ref result 'date #f)
       #:stylesheets (hash-ref result 'stylesheets (list))
       #:scripts (hash-ref result 'scripts (list))
       #:on-load (hash-ref result 'on-load #f)
       #:error-code (hash-ref result 'error-code 200))
     param)))
